@@ -32,8 +32,8 @@ i.e. networks of nodes connected via edges.
 
 __author__ = "Richard Gast, Daniel Rose"
 __status__ = "Development"
-__version__ = "0.9.6"
+__version__ = "0.11.2"
 
 
-class PyRatesException(Exception):
-    pass
+from pyrates.utility import integrate, grid_search, clear_frontend_caches, clear
+from pyrates.frontend import CircuitTemplate, NodeTemplate, EdgeTemplate, OperatorTemplate, save
