@@ -1,4 +1,3 @@
-
 # -*- coding: utf-8 -*-
 #
 #
@@ -37,6 +36,8 @@ PyRates backend module including the following sub-modules:
     Interface to the backends available in PyRates.
 """
 
-from .computegraph import ComputeGraph
-from .parser import ExpressionParser, parse_dict
-from .funcs import *
+from .parser import parse_equations
+
+
+class PyRatesException(Exception):
+    pass
