@@ -27,31 +27,80 @@
 # Richard Gast and Daniel Rose et. al. in preparation
 """
 """
-from copy import copy, deepcopy
-from typing import Iterator
-
-import numpy as np
+from typing import Iterator, Callable
 
 from pyrates.ir.abc import AbstractBaseIR
-from pyrates.ir.operator_graph import OperatorGraph, VectorizedOperatorGraph, cache_op_graph
+from pyrates.ir.operator_graph import OperatorGraph, VectorizedOperatorGraph
+from pyrates.backend.parser import get_unique_label
 
 __author__ = "Daniel Rose"
 __status__ = "Development"
 
+node_cache = {}
+op_cache = {}
+node_labels = []
+
+
+def clear_ir_caches():
+    node_cache.clear()
+    op_cache.clear()
+    node_labels.clear()
+
+
+def cache_func(label: str, operators: dict, values: dict = None, template: str = None, ir_class: Callable = None,
+               **kwargs):
+    if operators is None:
+        operators = {}
+
+    # compute hash from incoming operators. Different order of operators in input might lead to different hash.
+    op_graph = OperatorGraph(operators)
+    h = hash(op_graph)
+
+    changed_labels = dict()
+    vectorize = kwargs.pop('vectorize', True)
+    try:
+
+        # if vectorization is to be skipped, ignore cached IRs
+        if not vectorize:
+            raise KeyError
+
+        # extract node from cache
+        node = node_cache[h]
+
+        # change operator labels if necessary
+        for name, op in operators.items():
+            op_hash = hash(op)
+            for cached_name, cached_op in op_cache[h]:
+                if op_hash == hash(cached_op["operator"]):
+                    changed_labels[name] = cached_name
+                    for old_name, new_name in changed_labels.items():
+                        try:
+                            values[new_name] = values.pop(old_name)
+                        except (AttributeError, KeyError):
+                            pass
+
+        # extend cached node
+        node.extend(NodeIR(label, operators=op_graph, values=values, template=template), **kwargs)
+
+    except KeyError:
+
+        # create new node
+        op_cache[h] = op_graph
+        if label in node_labels:
+            label = get_unique_label(label, node_labels)
+        node = ir_class(label, operators=op_graph, values=values, template=template, **kwargs)
+        node_cache[h] = node
+        node_labels.append(label)
+
+    return node, changed_labels
+
 
 class NodeIR(AbstractBaseIR):
     __slots__ = ["_op_graph", "values"]
 
-    def __init__(self, operators: dict = None, values: dict = None, template: str = None):
-        super().__init__(template)
-        self._op_graph, changed_labels = cache_op_graph(OperatorGraph)(operators)
-        # ToDo: Move caching function to NodeIR instead of using a decorator, for clarity
-        try:
-            for old_name, new_name in changed_labels.items():
-                values[new_name] = values.pop(old_name)
-        except AttributeError:
-            pass
-
+    def __init__(self, label: str, operators: OperatorGraph, values: dict = None, template: str = None):
+        super().__init__(label, template)
+        self._op_graph = operators
         self.values = values
 
     @property
@@ -78,22 +127,16 @@ class NodeIR(AbstractBaseIR):
 class VectorizedNodeIR(AbstractBaseIR):
     """Alternate version of NodeIR that takes a full NodeIR as input and creates a vectorized form of it."""
 
-    __slots__ = ["op_graph", "_length"]
+    __slots__ = ["op_graph", "length"]
 
-    def __init__(self, node_ir: NodeIR):
+    def __init__(self, label: str, operators: OperatorGraph, values: dict = None, template: str = None):
 
-        super().__init__(node_ir.template)
-        self.op_graph = VectorizedOperatorGraph(node_ir.op_graph, node_ir.values)
-        values = {}
-        # reformat all values to be lists of themselves (adding an outer vector dimension)
-        # if len(node_ir.op_graph) == 0:
-        #     op_key, data = next(iter(self.op_graph.node(data=True)))
-        #     for var in data["variables"]:
-        #         values[op_key] = {var: [0.]}
-        # else:
+        super().__init__(label, template)
+
+        self.op_graph = VectorizedOperatorGraph(operators, values=values)
 
         # save current length of this node vector.
-        self._length = 1
+        self.length = 1
 
     def getitem_from_iterator(self, key: str, key_iter: Iterator[str]):
         """Alias for self.op_graph.getitem_from_iterator"""
@@ -126,7 +169,7 @@ class VectorizedNodeIR(AbstractBaseIR):
         # add values to respective lists in collapsed node
         self.op_graph.append_values(node.values)
 
-        self._length += 1
+        self.length += 1
 
     def __len__(self):
         """Returns size of this vector node as recorded in self._length.
@@ -135,7 +178,7 @@ class VectorizedNodeIR(AbstractBaseIR):
         -------
         self._length
         """
-        return self._length
+        return self.length
 
     def add_op(self, op_key: str, inputs: dict, output: str, equations: list, variables: dict):
         """Wrapper for internal `op_graph.add_operator` that adds any values to node-level values dictionary for quick
